@@ -292,8 +292,8 @@ class OWxcrystal(XoppyWidget):
         elif self.MOSAIC == 2 or self.MOSAIC == 3: #bent ML/PP
             self.ASYMMETRY_ANGLE = congruence.checkNumber(self.ASYMMETRY_ANGLE, "Asymmetry angle")
             self.THICKNESS = congruence.checkStrictlyPositiveNumber(self.THICKNESS, "Crystal thickness")
-            self.RSAG = congruence.checkNumber(self.RSAG, "R Sagittal")
-            self.RMER = congruence.checkNumber(self.RMER, "R meridional")
+            self.RSAG = congruence.checkStrictlyPositiveNumber(self.RSAG, "R Sagittal")
+            self.RMER = congruence.checkStrictlyPositiveNumber(self.RMER, "R meridional")
 
             if self.ANISOTROPY == 0:
                 self.POISSON = congruence.checkStrictlyPositiveNumber(self.POISSON, "Poisson Ratio")
