@@ -6,13 +6,22 @@ from PyQt5.QtGui import QIntValidator, QDoubleValidator
 from orangewidget import gui
 from oasys.widgets import gui as oasysgui, congruence
 from orangewidget.settings import Setting
-from srxraylib.sources import srfunc
+
 from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget
+from orangecontrib.xoppy.util.xoppy_bm_wiggler import xoppy_calc_bm
+
 from oasys.widgets.exchange import DataExchangeObject
 
 # TODO show flux(psi) and 2D plot
+from syned.widget.widget_decorator import WidgetDecorator
 
-class OWbm(XoppyWidget):
+from syned.storage_ring.magnetic_structures.bending_magnet import BendingMagnet
+import syned.beamline.beamline as synedb
+
+
+import scipy.constants as codata
+
+class OWbm(XoppyWidget, WidgetDecorator):
     name = "BM"
     id = "orange.widgets.databm"
     description = "Bending Magnet Spectrum"
@@ -21,6 +30,8 @@ class OWbm(XoppyWidget):
     category = ""
     keywords = ["xoppy", "bm"]
 
+    inputs = WidgetDecorator.syned_input_data()
+
     TYPE_CALC = Setting(0)
     MACHINE_NAME = Setting("ESRF bending magnet")
     RB_CHOICE = Setting(0)
@@ -40,6 +51,10 @@ class OWbm(XoppyWidget):
     PSI_NPOINTS = Setting(500)
     FILE_DUMP = Setting(0)
 
+    def __init__(self):
+        super().__init__(show_script_tab=True)
+
+
     def build_gui(self):
 
         box = oasysgui.widgetBox(self.controlArea, self.name + " Input Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-5)
@@ -57,7 +72,7 @@ class OWbm(XoppyWidget):
         
         #widget index 1 
         idx += 1 
-        box1 = gui.widgetBox(box) 
+        self.id_MACHINE_NAME = box1 = gui.widgetBox(box)
         oasysgui.lineEdit(box1, self, "MACHINE_NAME",
                      label=self.unitLabels()[idx], addSpace=False, orientation="horizontal")
         self.show_at(self.unitFlags()[idx], box1) 
@@ -74,7 +89,7 @@ class OWbm(XoppyWidget):
         #widget index 3 
         idx += 1 
         box1 = gui.widgetBox(box) 
-        oasysgui.lineEdit(box1, self, "MACHINE_R_M",
+        self.id_MACHINE_R_M = oasysgui.lineEdit(box1, self, "MACHINE_R_M",
                      label=self.unitLabels()[idx], addSpace=False,
                     valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
         self.show_at(self.unitFlags()[idx], box1) 
@@ -82,7 +97,7 @@ class OWbm(XoppyWidget):
         #widget index 4 
         idx += 1 
         box1 = gui.widgetBox(box) 
-        oasysgui.lineEdit(box1, self, "BFIELD_T",
+        self.id_BFIELD_T = oasysgui.lineEdit(box1, self, "BFIELD_T",
                      label=self.unitLabels()[idx], addSpace=False,
                     valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
         self.show_at(self.unitFlags()[idx], box1) 
@@ -90,7 +105,7 @@ class OWbm(XoppyWidget):
         #widget index 5 
         idx += 1 
         box1 = gui.widgetBox(box) 
-        oasysgui.lineEdit(box1, self, "BEAM_ENERGY_GEV",
+        self.id_BEAM_ENERGY_GEV = oasysgui.lineEdit(box1, self, "BEAM_ENERGY_GEV",
                      label=self.unitLabels()[idx], addSpace=False,
                     valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
         self.show_at(self.unitFlags()[idx], box1) 
@@ -98,7 +113,7 @@ class OWbm(XoppyWidget):
         #widget index 6 
         idx += 1 
         box1 = gui.widgetBox(box) 
-        oasysgui.lineEdit(box1, self, "CURRENT_A",
+        self.id_CURRENT_A = oasysgui.lineEdit(box1, self, "CURRENT_A",
                      label=self.unitLabels()[idx], addSpace=False,
                     valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
         self.show_at(self.unitFlags()[idx], box1) 
@@ -195,10 +210,22 @@ class OWbm(XoppyWidget):
         self.show_at(self.unitFlags()[idx], box1)
 
     def unitLabels(self):
-         return ['Type of calculation','Machine name','B from:','Machine Radius [m]','Magnetic Field [T]','Beam energy [GeV]','Beam Current [A]','Horizontal div Theta [mrad]','Psi (vertical div) for energy spectra','Min Photon Energy [eV]','Max Photon Energy [eV]','Number of energy points','Separation between energy points','Max Psi[mrad] for angular plots','Psi min [mrad]','Psi max [mrad]','Number of Psi points','Dump file']
+         return ['Type of calculation','Machine name',
+                 'B from:','Machine Radius [m]','Magnetic Field [T]',
+                 'Beam energy [GeV]','Beam Current [A]',
+                 'Horizontal div Theta [mrad]','Psi (vertical div) for energy spectra',
+                 'Min Photon Energy [eV]','Max Photon Energy [eV]','Number of points','Separation between energy points',
+                 'Max Psi[mrad] for angular plots','Psi min [mrad]','Psi max [mrad]','Number of Psi points',
+                 'Dump file']
 
     def unitFlags(self):
-         return ['True','True','True','self.RB_CHOICE  ==  0','self.RB_CHOICE  ==  1','True','True','True','True','True','True','True','True','True','self.VER_DIV  >=  2','self.VER_DIV  ==  2','self.VER_DIV  ==  2','True']
+         return ['True','True',
+                 'True','self.RB_CHOICE  ==  0','self.RB_CHOICE  ==  1',
+                 'True','True',
+                 'True','self.TYPE_CALC == 0',
+                 'True','True','True','self.TYPE_CALC == 0',
+                 'True','self.VER_DIV  >=  2','self.VER_DIV  ==  2','self.VER_DIV  ==  2',
+                 'True']
 
     def set_TYPE_CALC(self):
 
@@ -239,7 +266,9 @@ class OWbm(XoppyWidget):
             self.PSI_MIN = congruence.checkNumber(self.PSI_MIN, "Min Photon Energy")
 
     def plot_results(self, calculated_data, progressBarValue=80):
-        if self.TYPE_CALC != 3: super().plot_results(calculated_data, progressBarValue)
+        if self.TYPE_CALC != 3:
+            super().plot_results(calculated_data, progressBarValue)
+            self.tabs.setCurrentIndex(len(self.getTitles())-1)
         elif not self.view_type == 0:
             if not calculated_data is None:
                 self.view_type_combo.setEnabled(False)
@@ -255,19 +284,17 @@ class OWbm(XoppyWidget):
                                      a,
                                      energy_ev,
                                      0, 0,
-                                     xtitle="Angle [mrad]",
+                                     xtitle="Psi [mrad]",
                                      ytitle="Photon energy [eV]",
-                                     title="Flux [photons/s/0.1%bw/mrad]")
+                                     title="Flux [photons/s/0.1%bw]")
 
-                    self.plot_data2D(fm*srfunc.codata_ec*1e3,
+                    self.plot_data2D(fm*codata.e*1e3,
                                      a,
                                      energy_ev,
                                      1, 1,
-                                     xtitle="Angle [mrad]",
+                                     xtitle="Psi [mrad]",
                                      ytitle="Photon energy [eV]",
-                                     title="Spectral power [W/eV/mrad]")
-
-                    self.tabs.setCurrentIndex(1)
+                                     title="Spectral power [W/eV]")
                 except Exception as e:
                     self.view_type_combo.setEnabled(True)
 
@@ -277,8 +304,9 @@ class OWbm(XoppyWidget):
             else:
                 raise Exception("Empty Data")
 
+
     def do_xoppy_calculation(self):
-        return xoppy_calc_bm(TYPE_CALC=self.TYPE_CALC,
+        a6_T, fm, a, energy_ev =  xoppy_calc_bm(TYPE_CALC=self.TYPE_CALC,
                              MACHINE_NAME=self.MACHINE_NAME,
                              RB_CHOICE=self.RB_CHOICE,
                              MACHINE_R_M=self.MACHINE_R_M,
@@ -297,6 +325,70 @@ class OWbm(XoppyWidget):
                              PSI_NPOINTS=self.PSI_NPOINTS,
                              FILE_DUMP=self.FILE_DUMP)
 
+        # write python script in standard output
+        dict_parameters = {
+            "TYPE_CALC"       : self.TYPE_CALC,
+            "MACHINE_NAME"    : self.MACHINE_NAME,
+            "RB_CHOICE"       : self.RB_CHOICE,
+            "MACHINE_R_M"     : self.MACHINE_R_M,
+            "BFIELD_T"        : self.BFIELD_T,
+            "BEAM_ENERGY_GEV" : self.BEAM_ENERGY_GEV,
+            "CURRENT_A"       : self.CURRENT_A,
+            "HOR_DIV_MRAD"    : self.HOR_DIV_MRAD,
+            "VER_DIV"         : self.VER_DIV,
+            "PHOT_ENERGY_MIN" : self.PHOT_ENERGY_MIN,
+            "PHOT_ENERGY_MAX" : self.PHOT_ENERGY_MAX,
+            "NPOINTS"         : self.NPOINTS,
+            "LOG_CHOICE"      : self.LOG_CHOICE,
+            "PSI_MRAD_PLOT"   : self.PSI_MRAD_PLOT,
+            "PSI_MIN"         : self.PSI_MIN,
+            "PSI_MAX"         : self.PSI_MAX,
+            "PSI_NPOINTS"     : self.PSI_NPOINTS,
+            "FILE_DUMP"       : self.FILE_DUMP,
+            }
+
+        self.xoppy_script.set_code(self.script_template().format_map(dict_parameters))
+
+        return a6_T, fm, a, energy_ev
+
+
+
+    def script_template(self):
+        return """
+#
+# script to make the calculations (created by XOPPY:bm)
+#
+from orangecontrib.xoppy.util.xoppy_bm_wiggler import xoppy_calc_bm
+
+# TYPE_CALC: 
+# 0: 'Energy or Power spectra'
+# 1: 'Angular distribution (all wavelengths)'
+# 2: 'Angular distribution (one wavelength)'
+# 3: '2D flux and power (angular,energy) distribution'
+#
+a6_T, fm, a, energy_ev =  xoppy_calc_bm(
+    TYPE_CALC={TYPE_CALC},
+    MACHINE_NAME="{MACHINE_NAME}",
+    RB_CHOICE={RB_CHOICE},
+    MACHINE_R_M={MACHINE_R_M},
+    BFIELD_T={BFIELD_T},
+    BEAM_ENERGY_GEV={BEAM_ENERGY_GEV},
+    CURRENT_A={CURRENT_A},
+    HOR_DIV_MRAD={HOR_DIV_MRAD},
+    VER_DIV={VER_DIV},
+    PHOT_ENERGY_MIN={PHOT_ENERGY_MIN},
+    PHOT_ENERGY_MAX={PHOT_ENERGY_MAX},
+    NPOINTS={NPOINTS},
+    LOG_CHOICE={LOG_CHOICE},
+    PSI_MRAD_PLOT={PSI_MRAD_PLOT},
+    PSI_MIN={PSI_MIN},
+    PSI_MAX={PSI_MAX},
+    PSI_NPOINTS={PSI_NPOINTS},
+    FILE_DUMP=True) # writes output to bm.spec
+#
+# end script
+#
+"""
     def add_specific_content_to_calculated_data(self, calculated_data):
         calculated_data.add_content("is_log_plot", self.LOG_CHOICE)
         calculated_data.add_content("calculation_type", self.TYPE_CALC)
@@ -371,154 +463,39 @@ class OWbm(XoppyWidget):
         elif self.TYPE_CALC == 3:
             return []
 
-# --------------------------------------------------------------------------------------------
-# --------------------------------------------------------------------------------------------
+    def receive_syned_data(self, data):
 
+        if isinstance(data, synedb.Beamline):
+            if not data is None:
+                if not data._light_source is None and isinstance(data._light_source._magnetic_structure, BendingMagnet):
+                    light_source = data._light_source
 
-def xoppy_calc_bm(MACHINE_NAME="ESRF bending magnet",RB_CHOICE=0,MACHINE_R_M=25.0,BFIELD_T=0.8,\
-                  BEAM_ENERGY_GEV=6.04,CURRENT_A=0.1,HOR_DIV_MRAD=1.0,VER_DIV=0,\
-                  PHOT_ENERGY_MIN=100.0,PHOT_ENERGY_MAX=100000.0,NPOINTS=500,LOG_CHOICE=1,\
-                  PSI_MRAD_PLOT=1.0,PSI_MIN=-1.0,PSI_MAX=1.0,PSI_NPOINTS=500,TYPE_CALC=0,FILE_DUMP=0):
-    print("Inside xoppy_calc_bm. ")
+                    self.MACHINE_NAME = light_source._name
+                    self.BEAM_ENERGY_GEV = light_source._electron_beam._energy_in_GeV
+                    self.CURRENT_A = light_source._electron_beam._current
+                    self.BFIELD_T = light_source._magnetic_structure._magnetic_field
+                    self.MACHINE_R_M = light_source._magnetic_structure._radius
 
-    # electron energy in GeV
-    gamma = BEAM_ENERGY_GEV*1e3 / srfunc.codata_mee
-
-    r_m = MACHINE_R_M      # magnetic radius in m
-    if RB_CHOICE == 1:
-        r_m = srfunc.codata_me * srfunc.codata_c / srfunc.codata_ec / BFIELD_T * numpy.sqrt(gamma * gamma - 1)
-
-    # calculate critical energy in eV
-    ec_m = 4.0*numpy.pi*r_m/3.0/numpy.power(gamma,3) # wavelength in m
-    ec_ev = srfunc.m2ev / ec_m
-
-    fm = None
-    a = None
-    energy_ev = None
-
-    if TYPE_CALC == 0:
-        if LOG_CHOICE == 0:
-            energy_ev = numpy.linspace(PHOT_ENERGY_MIN,PHOT_ENERGY_MAX,NPOINTS) # photon energy grid
+                    self.set_enabled(False)
+                else:
+                    self.set_enabled(True)
+                    raise ValueError("Syned data not correct")
         else:
-            energy_ev = numpy.logspace(numpy.log10(PHOT_ENERGY_MIN),numpy.log10(PHOT_ENERGY_MAX),NPOINTS) # photon energy grid
+            self.set_enabled(True)
 
-        a5 = srfunc.sync_ene(VER_DIV, energy_ev, ec_ev=ec_ev, polarization=0, \
-                             e_gev=BEAM_ENERGY_GEV, i_a=CURRENT_A, hdiv_mrad=HOR_DIV_MRAD, \
-                             psi_min=PSI_MIN, psi_max=PSI_MAX, psi_npoints=PSI_NPOINTS)
-
-        a5par = srfunc.sync_ene(VER_DIV, energy_ev, ec_ev=ec_ev, polarization=1, \
-                                e_gev=BEAM_ENERGY_GEV, i_a=CURRENT_A, hdiv_mrad=HOR_DIV_MRAD, \
-                                psi_min=PSI_MIN, psi_max=PSI_MAX, psi_npoints=PSI_NPOINTS)
-
-        a5per = srfunc.sync_ene(VER_DIV, energy_ev, ec_ev=ec_ev, polarization=2, \
-                                e_gev=BEAM_ENERGY_GEV, i_a=CURRENT_A, hdiv_mrad=HOR_DIV_MRAD, \
-                                psi_min=PSI_MIN, psi_max=PSI_MAX, psi_npoints=PSI_NPOINTS)
-
-        if VER_DIV == 0:
-            coltitles=['Photon Energy [eV]','Photon Wavelength [A]','E/Ec','Flux_spol/Flux_total','Flux_ppol/Flux_total','Flux[Phot/sec/0.1%bw]','Power[Watts/eV]']
-            title='integrated in Psi,'
-        if VER_DIV == 1:
-            coltitles=['Photon Energy [eV]','Photon Wavelength [A]','E/Ec','Flux_spol/Flux_total','Flux_ppol/Flux_total','Flux[Phot/sec/0.1%bw/mrad(Psi)]','Power[Watts/eV/mrad(Psi)]']
-            title='at Psi=0,'
-        if VER_DIV == 2:
-            coltitles=['Photon Energy [eV]','Photon Wavelength [A]','E/Ec','Flux_spol/Flux_total','Flux_ppol/Flux_total','Flux[Phot/sec/0.1%bw]','Power[Watts/eV]']
-            title='in Psi=[%e,%e]'%(PSI_MIN,PSI_MAX)
-        if VER_DIV == 3:
-            coltitles=['Photon Energy [eV]','Photon Wavelength [A]','E/Ec','Flux_spol/Flux_total','Flux_ppol/Flux_total','Flux[Phot/sec/0.1%bw/mrad(Psi)]','Power[Watts/eV/mrad(Psi)]']
-            title='at Psi=%e mrad'%(PSI_MIN)
-
-        a6=numpy.zeros((7,len(energy_ev)))
-        a1 = energy_ev
-        a6[0,:] = (a1)
-        a6[1,:] = srfunc.m2ev * 1e10 / (a1)
-        a6[2,:] = (a1)/ec_ev # E/Ec
-        a6[3,:] = numpy.array(a5par)/numpy.array(a5)
-        a6[4,:] = numpy.array(a5per)/numpy.array(a5)
-        a6[5,:] = numpy.array(a5)
-        a6[6,:] = numpy.array(a5)*1e3 * srfunc.codata_ec
-
-    if TYPE_CALC == 1:  # angular distributions over over all energies
-        angle_mrad = numpy.linspace(-PSI_MRAD_PLOT, +PSI_MRAD_PLOT,NPOINTS) # angle grid
-
-        a6 = numpy.zeros((6,NPOINTS))
-        a6[0,:] = angle_mrad # angle in mrad
-        a6[1,:] = angle_mrad*gamma/1e3 # Psi[rad]*Gamma
-        a6[2,:] = srfunc.sync_f(angle_mrad * gamma / 1e3)
-        a6[3,:] = srfunc.sync_f(angle_mrad * gamma / 1e3, polarization=1)
-        a6[4,:] = srfunc.sync_f(angle_mrad * gamma / 1e3, polarization=2)
-        a6[5,:] = srfunc.sync_ang(0, angle_mrad, i_a=CURRENT_A, hdiv_mrad=HOR_DIV_MRAD, e_gev=BEAM_ENERGY_GEV, r_m=r_m)
-
-        coltitles=['Psi[mrad]','Psi[rad]*Gamma','F','F s-pol','F p-pol','Power[Watts/mrad(Psi)]']
-
-    if TYPE_CALC == 2:  # angular distributions at a single energy
-        angle_mrad = numpy.linspace(-PSI_MRAD_PLOT, +PSI_MRAD_PLOT,NPOINTS) # angle grid
-
-        a6 = numpy.zeros((7,NPOINTS))
-        a6[0,:] = angle_mrad # angle in mrad
-        a6[1,:] = angle_mrad*gamma/1e3 # Psi[rad]*Gamma
-        a6[2,:] = srfunc.sync_f(angle_mrad * gamma / 1e3)
-        a6[3,:] = srfunc.sync_f(angle_mrad * gamma / 1e3, polarization=1)
-        a6[4,:] = srfunc.sync_f(angle_mrad * gamma / 1e3, polarization=2)
-        tmp = srfunc.sync_ang(1, angle_mrad, energy=PHOT_ENERGY_MIN, i_a=CURRENT_A, hdiv_mrad=HOR_DIV_MRAD, e_gev=BEAM_ENERGY_GEV, ec_ev=ec_ev)
-        tmp.shape = -1
-        a6[5,:] = tmp
-        a6[6,:] = a6[5,:] * srfunc.codata_ec * 1e3
-
-        coltitles=['Psi[mrad]','Psi[rad]*Gamma','F','F s-pol','F p-pol','Flux[Ph/sec/0.1%bw/mrad(Psi)]','Power[Watts/eV/mrad(Psi)]']
-
-    if TYPE_CALC == 3:  # angular,energy distributions flux
-        angle_mrad = numpy.linspace(-PSI_MRAD_PLOT, +PSI_MRAD_PLOT,NPOINTS) # angle grid
-
-        if LOG_CHOICE == 0:
-            energy_ev = numpy.linspace(PHOT_ENERGY_MIN,PHOT_ENERGY_MAX,NPOINTS) # photon energy grid
+    def set_enabled(self,value):
+        if value == True:
+                self.id_MACHINE_NAME.setEnabled(True)
+                self.id_BEAM_ENERGY_GEV.setEnabled(True)
+                self.id_CURRENT_A.setEnabled(True)
+                self.id_BFIELD_T.setEnabled(True)
+                self.id_MACHINE_R_M.setEnabled(True)
         else:
-            energy_ev = numpy.logspace(numpy.log10(PHOT_ENERGY_MIN),numpy.log10(PHOT_ENERGY_MAX),NPOINTS) # photon energy grid
-
-        # fm[angle,energy]
-        fm = srfunc.sync_ene(4, energy_ev, ec_ev=ec_ev, e_gev=BEAM_ENERGY_GEV, i_a=CURRENT_A, \
-                                      hdiv_mrad=HOR_DIV_MRAD, psi_min=PSI_MIN, psi_max=PSI_MAX, psi_npoints=PSI_NPOINTS)
-
-        a = numpy.linspace(PSI_MIN,PSI_MAX,PSI_NPOINTS)
-
-        a6 = numpy.zeros((4,len(a)*len(energy_ev)))
-        ij = -1
-        for i in range(len(a)):
-            for j in range(len(energy_ev)):
-                ij += 1
-                a6[0,ij] = a[i]
-                a6[1,ij] = energy_ev[j]
-                a6[2,ij] = fm[i,j] * srfunc.codata_ec * 1e3
-                a6[3,ij] = fm[i,j]
-
-        coltitles=['Psi [mrad]','Photon Energy [eV]','Power [Watts/eV/mrad(Psi)]','Flux [Ph/sec/0.1%bw/mrad(Psi)]']
-
-    # write spec file
-    ncol = len(coltitles)
-    npoints = len(a6[0,:])
-
-    if FILE_DUMP == 1:
-        outFile = "bm.spec"
-        f = open(outFile,"w")
-        f.write("#F "+outFile+"\n")
-        f.write("\n")
-        f.write("#S 1 bm results\n")
-        f.write("#N %d\n"%(ncol))
-        f.write("#L")
-        for i in range(ncol):
-            f.write("  "+coltitles[i])
-        f.write("\n")
-
-        for i in range(npoints):
-                f.write((" %e "*ncol+"\n")%(tuple(a6[:,i].tolist())))
-        f.close()
-        print("File written to disk: " + outFile)
-
-    if TYPE_CALC == 0:
-        if LOG_CHOICE == 0:
-            print("\nPower from integral of spectrum: %15.3f W"%(a5.sum() * 1e3*srfunc.codata_ec * (energy_ev[1]-energy_ev[0])))
-
-    return a6.T, fm, a, energy_ev
-
+                self.id_MACHINE_NAME.setEnabled(False)
+                self.id_BEAM_ENERGY_GEV.setEnabled(False)
+                self.id_CURRENT_A.setEnabled(False)
+                self.id_BFIELD_T.setEnabled(False)
+                self.id_MACHINE_R_M.setEnabled(False)
 
 
 if __name__ == "__main__":
